﻿using MediatR;
using Microsoft.Extensions.Logging;
using NetworkResurrector.Api.Domain.Repositories;
using NetworkResurrector.Api.PublishedLanguage.Commands;
using NetworkResurrector.Api.PublishedLanguage.Events;
using NetworkResurrector.Server.Wrapper.Services;
using System.Threading;
using System.Threading.Tasks;

namespace NetworkResurrector.Api.Application.CommandHandlers
{
    public class ShutdownMachineHandler : IRequestHandler<ShutdownMachine, MachineShutdown>
    {
        private readonly ILogger<ShutdownMachineHandler> _logger;
        private readonly IResurrectorService _resurrectorService;
        private readonly INetworkRepository _repository;

        public ShutdownMachineHandler(ILogger<ShutdownMachineHandler> logger, IResurrectorService resurrectorService, INetworkRepository repository)
        {
            _logger=logger;
            _resurrectorService=resurrectorService;
            _repository=repository;
        }

        public async Task<MachineShutdown> Handle(ShutdownMachine command, CancellationToken cancellationToken)
        {
            _logger.LogDebug($"Start shutdown machine {command.MachineId}");
            var machine = await _repository.GetMachine(command.MachineId);

            //log activity
            var pingResult = await _resurrectorService.Shutdown(machine.IPv4Address ?? machine.MachineName);
            var result = new MachineShutdown(pingResult.Success, pingResult.Status);
            _logger.LogDebug($"Machine {command.MachineId} shutdown finished. Success: {result.Success}; Status: {result.Status}");

            return result;
        }
    }
}
